package java.net;

import java.util.Iterator;
import sun.net.SocksProxy;
import sun.net.www.ParseUtil;
import java.io.UnsupportedEncodingException;
import sun.security.action.GetPropertyAction;
import java.security.PrivilegedAction;
import java.io.BufferedOutputStream;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.io.IOException;
import java.security.PrivilegedExceptionAction;
import java.io.OutputStream;
import java.io.InputStream;

class SocksSocketImpl extends PlainSocketImpl implements SocksConsts
{
    private String server;
    private int serverPort;
    private InetSocketAddress external_address;
    private boolean useV4;
    private Socket cmdsock;
    private InputStream cmdIn;
    private OutputStream cmdOut;
    private boolean applicationSetProxy;
    
    SocksSocketImpl() {
        super(false);
        this.server = null;
        this.serverPort = 1080;
        this.useV4 = false;
        this.cmdsock = null;
        this.cmdIn = null;
        this.cmdOut = null;
    }
    
    SocksSocketImpl(String server, int n) {
        super(false);
        this.server = null;
        this.serverPort = 1080;
        this.useV4 = false;
        this.cmdsock = null;
        this.cmdIn = null;
        this.cmdOut = null;
        this.server = server;
        this.serverPort = ((n == -1) ? 1080 : n);
    }
    
    SocksSocketImpl(Proxy proxy) {
        super(false);
        this.server = null;
        this.serverPort = 1080;
        this.useV4 = false;
        this.cmdsock = null;
        this.cmdIn = null;
        this.cmdOut = null;
        SocketAddress address = proxy.address();
        if ((address instanceof InetSocketAddress)) {
            InetSocketAddress inetSocketAddress = (InetSocketAddress)address;
            this.server = inetSocketAddress.getHostString();
            this.serverPort = inetSocketAddress.getPort();
        }
    }
    
    void setV4() {
        this.useV4 = true;
    }
    
    private synchronized void privilegedConnect(final String s, final int n, final int n2) throws IOException {
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                @Override
                public Void run() throws IOException {
                    SocksSocketImpl.access$000(SocksSocketImpl.this, s, n, n2);
                    SocksSocketImpl.this.cmdIn = SocksSocketImpl.this.getInputStream();
                    SocksSocketImpl.this.cmdOut = SocksSocketImpl.this.getOutputStream();
                    return null;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw (IOException)ex.getException();
        }
    }
    
    private void superConnectServer(String s, int n, int n2) throws IOException {
        super.connect(new InetSocketAddress(s, n), n2);
    }
    
    private static int remainingMillis(long n) throws IOException {
        if (n == 0L) {
            return 0;
        }
        long n2 = n - System.currentTimeMillis();
        if (n2 > 0L) {
            return (int)n2;
        }
        throw new SocketTimeoutException();
    }
    
    private int readSocksReply(InputStream inputStream, byte[] array) throws IOException {
        return this.readSocksReply(inputStream, array, 0L);
    }
    
    private int readSocksReply(InputStream inputStream, byte[] array, long n) throws IOException {
        int length = array.length;
        int n2 = 0;
        int n3 = 0;
        while (n2 < length && n3 < 3) {
            int read;
            try {
                read = ((SocketInputStream)inputStream).read(array, n2, length - n2, remainingMillis(n));
            }
            catch (SocketTimeoutException ex) {
                throw new SocketTimeoutException("Connect timed out");
            }
            if (read < 0) {
                throw new SocketException("Malformed reply from SOCKS server");
            }
            n2 += read;
            ++n3;
        }
        return n2;
    }
    
    private boolean authenticate(byte b, InputStream inputStream, BufferedOutputStream bufferedOutputStream) throws IOException {
        return this.authenticate(b, inputStream, bufferedOutputStream, 0L);
    }
    
    private boolean authenticate(byte b, InputStream inputStream, BufferedOutputStream bufferedOutputStream, long n) throws IOException {
        if (b == 0) {
            return true;
        }
        if (b == 2) {
            String s = null;
            PasswordAuthentication passwordAuthentication = (PasswordAuthentication)AccessController.doPrivileged(new PrivilegedAction<PasswordAuthentication>() {
                
                @Override
                public PasswordAuthentication run() {
                    return Authenticator.requestPasswordAuthentication(SocksSocketImpl.this.server, addr, SocksSocketImpl.this.serverPort, "SOCKS5", "SOCKS authentication", null);
                }
            });
            String userName;
            if (passwordAuthentication != null) {
                userName = passwordAuthentication.getUserName();
                s = new String(passwordAuthentication.getPassword());
            }
            else {
                userName = (String)AccessController.doPrivileged(new GetPropertyAction("user.name"));
            }
            if (userName == null) {
                return false;
            }
            bufferedOutputStream.write(1);
            bufferedOutputStream.write(userName.length());
            try {
                bufferedOutputStream.write(userName.getBytes("ISO-8859-1"));
            }
            catch (UnsupportedEncodingException ex) {
                assert false;
            }
            if (s != null) {
                bufferedOutputStream.write(s.length());
                try {
                    bufferedOutputStream.write(s.getBytes("ISO-8859-1"));
                }
                catch (UnsupportedEncodingException ex2) {
                    assert false;
                }
            }
            else {
                bufferedOutputStream.write(0);
            }
            bufferedOutputStream.flush();
            byte[] array = new byte[2];
            if (this.readSocksReply(inputStream, array, n) != 2 || array[1] != 0) {
                bufferedOutputStream.close();
                inputStream.close();
                return false;
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private void connectV4(InputStream inputStream, OutputStream outputStream, InetSocketAddress external_address, long n) throws IOException {
        if (!(external_address.getAddress() instanceof Inet4Address)) {
            throw new SocketException("SOCKS V4 requires IPv4 only addresses");
        }
        outputStream.write(4);
        outputStream.write(1);
        outputStream.write(external_address.getPort() >> 8 & 0xFF);
        outputStream.write(external_address.getPort() >> 0 & 0xFF);
        outputStream.write(external_address.getAddress().getAddress());
        String userName = this.getUserName();
        try {
            outputStream.write(userName.getBytes("ISO-8859-1"));
        }
        catch (UnsupportedEncodingException ex) {
            assert false;
        }
        outputStream.write(0);
        outputStream.flush();
        byte[] array = new byte[8];
        int socksReply = this.readSocksReply(inputStream, array, n);
        if (socksReply != 8) {
            throw new SocketException("Reply from SOCKS server has bad length: " + socksReply);
        }
        if (array[0] != 0 && array[0] != 4) {
            throw new SocketException("Reply from SOCKS server has bad version");
        }
        SocketException ex2 = null;
        switch (array[1]) {
            case 90: {
                this.external_address = external_address;
                break;
            }
            case 91: {
                ex2 = new SocketException("SOCKS request rejected");
                break;
            }
            case 92: {
                ex2 = new SocketException("SOCKS server couldn't reach destination");
                break;
            }
            case 93: {
                ex2 = new SocketException("SOCKS authentication failed");
                break;
            }
            default: {
                ex2 = new SocketException("Reply from SOCKS server contains bad status");
                break;
            }
        }
        if (ex2 != null) {
            inputStream.close();
            outputStream.close();
            throw ex2;
        }
        return;
    }
    
    @Override
    protected void connect(SocketAddress socketAddress, int n) throws IOException {
        long n3;
        if (n == 0) {
            n3 = 0L;
        }
        else {
            long n2 = System.currentTimeMillis() + (long)n;
            n3 = ((n2 < 0L) ? Long.MAX_VALUE : n2);
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (socketAddress == null || !(socketAddress instanceof InetSocketAddress)) {
            throw new IllegalArgumentException("Unsupported address type");
        }
        InetSocketAddress external_address = (InetSocketAddress)socketAddress;
        if (securityManager != null) {
            if (external_address.isUnresolved()) {
                securityManager.checkConnect(external_address.getHostName(), external_address.getPort());
            }
            else {
                securityManager.checkConnect(external_address.getAddress().getHostAddress(), external_address.getPort());
            }
        }
        if (this.server == null) {
            ProxySelector proxySelector = (ProxySelector)AccessController.doPrivileged(new PrivilegedAction<ProxySelector>(this) {
                @Override
                public ProxySelector run() {
                    return ProxySelector.getDefault();
                }
            });
            if (proxySelector == null) {
                super.connect(external_address, remainingMillis(n3));
                return;
            }
            String s = external_address.getHostString();
            if (external_address.getAddress() instanceof Inet6Address && !s.startsWith("[") && s.indexOf(":") >= 0) {
                s = "[" + s + "]";
            }
            Object o;
            try {
                o = new URI("socket://" + ParseUtil.encodePath(s) + ":" + external_address.getPort());
            }
            catch (URISyntaxException detailMessage) {
                assert false : detailMessage;
                o = null;
            }
            IOException ex2 = null;
            Iterator<Proxy> iterator = proxySelector.select(o).iterator();
            if (iterator == null || !iterator.hasNext()) {
                super.connect(external_address, remainingMillis(n3));
                return;
            }
            while (iterator.hasNext()) {
                Proxy obj = (Proxy)iterator.next();
                if (obj == null || obj.type() != Proxy.Type.SOCKS) {
                    super.connect(external_address, remainingMillis(n3));
                    return;
                }
                if (!(obj.address() instanceof InetSocketAddress)) {
                    throw new SocketException("Unknown address type for proxy: " + obj);
                }
                this.server = ((InetSocketAddress)obj.address()).getHostString();
                this.serverPort = ((InetSocketAddress)obj.address()).getPort();
                if (obj instanceof SocksProxy && ((SocksProxy)obj).protocolVersion() == 4) {
                    this.useV4 = true;
                }
                try {
                    this.privilegedConnect(this.server, this.serverPort, remainingMillis(n3));
                    break;
                }
                catch (IOException ex3) {
                    proxySelector.connectFailed(o, obj.address(), ex3);
                    this.server = null;
                    this.serverPort = -1;
                    ex2 = ex3;
                }
            }
            if (this.server == null) {
                throw new SocketException("Can't connect to SOCKS proxy:" + ex2.getMessage());
            }
        }
        else {
            try {
                this.privilegedConnect(this.server, this.serverPort, remainingMillis(n3));
            }
            catch (IOException ex) {
                throw new SocketException(ex.getMessage());
            }
        }
        BufferedOutputStream o2 = new BufferedOutputStream(this.cmdOut, 512);
        InputStream cmdIn = this.cmdIn;
        if (this.useV4) {
            if (external_address.isUnresolved()) {
                throw new UnknownHostException(external_address.toString());
            }
            this.connectV4(cmdIn, o2, external_address, n3);
            return;
        }
        else {
            o2.write(5);
            o2.write(2);
            o2.write(0);
            o2.write(2);
            o2.flush();
            byte[] array = new byte[2];
            if (this.readSocksReply(cmdIn, array, n3) != 2 || array[0] != 5) {
                if (external_address.isUnresolved()) {
                    throw new UnknownHostException(external_address.toString());
                }
                this.connectV4(cmdIn, o2, external_address, n3);
                return;
            }
            else {
                if (array[1] == -1) {
                    throw new SocketException("SOCKS : No acceptable methods");
                }
                if (!this.authenticate(array[1], cmdIn, o2, n3)) {
                    throw new SocketException("SOCKS : authentication failed");
                }
                o2.write(5);
                o2.write(1);
                o2.write(0);
                if (external_address.isUnresolved()) {
                    o2.write(3);
                    o2.write(external_address.getHostName().length());
                    try {
                        o2.write(external_address.getHostName().getBytes("ISO-8859-1"));
                    }
                    catch (UnsupportedEncodingException ex4) {
                        assert false;
                    }
                    o2.write(external_address.getPort() >> 8 & 0xFF);
                    o2.write(external_address.getPort() >> 0 & 0xFF);
                }
                else if ((external_address.getAddress() instanceof Inet6Address)) {
                    o2.write(4);
                    o2.write(external_address.getAddress().getAddress());
                    o2.write(external_address.getPort() >> 8 & 0xFF);
                    o2.write(external_address.getPort() >> 0 & 0xFF);
                }
                else {
                    o2.write(1);
                    o2.write(external_address.getAddress().getAddress());
                    o2.write(external_address.getPort() >> 8 & 0xFF);
                    o2.write(external_address.getPort() >> 0 & 0xFF);
                }
                o2.flush();
                byte[] array2 = new byte[4];
                if (this.readSocksReply(cmdIn, array2, n3) != 4) {
                    throw new SocketException("Reply from SOCKS server has bad length");
                }
                Object o3 = null;
                switch (array2[1]) {
                    case 0: {
                        switch (array2[3]) {
                            case 1: {
                                if (this.readSocksReply(cmdIn, new byte[4], n3) != 4) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                if (this.readSocksReply(cmdIn, new byte[2], n3) != 2) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                break;
                            }
                            case 3: {
                                int n4 = array2[1];
                                if (this.readSocksReply(cmdIn, new byte[n4], n3) != n4) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                if (this.readSocksReply(cmdIn, new byte[2], n3) != 2) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                break;
                            }
                            case 4: {
                                int n5 = array2[1];
                                if (this.readSocksReply(cmdIn, new byte[n5], n3) != n5) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                if (this.readSocksReply(cmdIn, new byte[2], n3) != 2) {
                                    throw new SocketException("Reply from SOCKS server badly formatted");
                                }
                                break;
                            }
                            default: {
                                o3 = new SocketException("Reply from SOCKS server contains wrong code");
                                break;
                            }
                        }
                        break;
                    }
                    case 1: {
                        o3 = new SocketException("SOCKS server general failure");
                        break;
                    }
                    case 2: {
                        o3 = new SocketException("SOCKS: Connection not allowed by ruleset");
                        break;
                    }
                    case 3: {
                        o3 = new SocketException("SOCKS: Network unreachable");
                        break;
                    }
                    case 4: {
                        o3 = new SocketException("SOCKS: Host unreachable");
                        break;
                    }
                    case 5: {
                        o3 = new SocketException("SOCKS: Connection refused");
                        break;
                    }
                    case 6: {
                        o3 = new SocketException("SOCKS: TTL expired");
                        break;
                    }
                    case 7: {
                        o3 = new SocketException("SOCKS: Command not supported");
                        break;
                    }
                    case 8: {
                        o3 = new SocketException("SOCKS: address type not supported");
                        break;
                    }
                }
                if (o3 != null) {
                    cmdIn.close();
                    o2.close();
                    throw o3;
                }
                this.external_address = external_address;
                return;
            }
        }
    }
    
    private void bindV4(InputStream inputStream, OutputStream outputStream, InetAddress inetAddress, int n) throws IOException {
        if (!(inetAddress instanceof Inet4Address)) {
            throw new SocketException("SOCKS V4 requires IPv4 only addresses");
        }
        super.bind(inetAddress, n);
        byte[] b = inetAddress.getAddress();
        if (inetAddress.isAnyLocalAddress()) {
            b = ((InetAddress)AccessController.doPrivileged(new PrivilegedAction<InetAddress>() {
                @Override
                public InetAddress run() {
                    return SocksSocketImpl.this.cmdsock.getLocalAddress();
                }
            })).getAddress();
        }
        outputStream.write(4);
        outputStream.write(2);
        outputStream.write(super.getLocalPort() >> 8 & 0xFF);
        outputStream.write(super.getLocalPort() >> 0 & 0xFF);
        outputStream.write(b);
        String userName = this.getUserName();
        try {
            outputStream.write(userName.getBytes("ISO-8859-1"));
        }
        catch (UnsupportedEncodingException ex) {
            assert false;
        }
        outputStream.write(0);
        outputStream.flush();
        byte[] array = new byte[8];
        int socksReply = this.readSocksReply(inputStream, array);
        if (socksReply != 8) {
            throw new SocketException("Reply from SOCKS server has bad length: " + socksReply);
        }
        if (array[0] != 0 && array[0] != 4) {
            throw new SocketException("Reply from SOCKS server has bad version");
        }
        Object o = null;
        switch (array[1]) {
            case 90: {
                this.external_address = new InetSocketAddress(inetAddress, n);
                break;
            }
            case 91: {
                o = new SocketException("SOCKS request rejected");
                break;
            }
            case 92: {
                o = new SocketException("SOCKS server couldn't reach destination");
                break;
            }
            case 93: {
                o = new SocketException("SOCKS authentication failed");
                break;
            }
            default: {
                o = new SocketException("Reply from SOCKS server contains bad status");
                break;
            }
        }
        if (o != null) {
            inputStream.close();
            outputStream.close();
            throw o;
        }
        return;
    }
    
    protected synchronized void socksBind(InetSocketAddress obj) throws IOException {
        if (this.socket != null) {
            return;
        }
        if (this.server == null) {
            ProxySelector proxySelector = (ProxySelector)AccessController.doPrivileged(new PrivilegedAction<ProxySelector>() {
                @Override
                public ProxySelector run() {
                    return ProxySelector.getDefault();
                }
            });
            if (proxySelector == null) {
                return;
            }
            String s = obj.getHostString();
            if (obj.getAddress() instanceof Inet6Address && !s.startsWith("[") && s.indexOf(":") >= 0) {
                s = "[" + s + "]";
            }
            Object o;
            try {
                o = new URI("serversocket://" + ParseUtil.encodePath(s) + ":" + obj.getPort());
            }
            catch (URISyntaxException detailMessage) {
                assert false : detailMessage;
                o = null;
            }
            Exception ex2 = null;
            Iterator<Proxy> iterator = proxySelector.select(o).iterator();
            if (iterator == null || !iterator.hasNext()) {
                return;
            }
            while (iterator.hasNext()) {
                Proxy obj2 = (Proxy)iterator.next();
                if (obj2 == null || obj2.type() != Proxy.Type.SOCKS) {
                    return;
                }
                if (!(obj2.address() instanceof InetSocketAddress)) {
                    throw new SocketException("Unknown address type for proxy: " + obj2);
                }
                this.server = ((InetSocketAddress)obj2.address()).getHostString();
                this.serverPort = ((InetSocketAddress)obj2.address()).getPort();
                if (obj2 instanceof SocksProxy && ((SocksProxy)obj2).protocolVersion() == 4) {
                    this.useV4 = true;
                }
                try {
                    AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                        @Override
                        public Void run() throws Exception {
                            SocksSocketImpl.this.cmdsock = new Socket(new PlainSocketImpl(false));
                            SocksSocketImpl.this.cmdsock.connect(new InetSocketAddress(SocksSocketImpl.this.server, SocksSocketImpl.this.serverPort));
                            SocksSocketImpl.this.cmdIn = SocksSocketImpl.this.cmdsock.getInputStream();
                            SocksSocketImpl.this.cmdOut = SocksSocketImpl.this.cmdsock.getOutputStream();
                            return null;
                        }
                    });
                }
                catch (Exception ex3) {
                    proxySelector.connectFailed(o, obj2.address(), new SocketException(ex3.getMessage()));
                    this.server = null;
                    this.serverPort = -1;
                    this.cmdsock = null;
                    ex2 = ex3;
                }
            }
            if (this.server == null || this.cmdsock == null) {
                throw new SocketException("Can't connect to SOCKS proxy:" + ex2.getMessage());
            }
        }
        else {
            try {
                AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                    @Override
                    public Void run() throws Exception {
                        SocksSocketImpl.this.cmdsock = new Socket(new PlainSocketImpl(false));
                        SocksSocketImpl.this.cmdsock.connect(new InetSocketAddress(SocksSocketImpl.this.server, SocksSocketImpl.this.serverPort));
                        SocksSocketImpl.this.cmdIn = SocksSocketImpl.this.cmdsock.getInputStream();
                        SocksSocketImpl.this.cmdOut = SocksSocketImpl.this.cmdsock.getOutputStream();
                        return null;
                    }
                });
            }
            catch (Exception ex) {
                throw new SocketException(ex.getMessage());
            }
        }
        BufferedOutputStream cmdOut = new BufferedOutputStream(this.cmdOut, 512);
        InputStream cmdIn = this.cmdIn;
        if (this.useV4) {
            this.bindV4(cmdIn, cmdOut, obj.getAddress(), obj.getPort());
            return;
        }
        cmdOut.write(5);
        cmdOut.write(2);
        cmdOut.write(0);
        cmdOut.write(2);
        cmdOut.flush();
        byte[] array = new byte[2];
        if (this.readSocksReply(cmdIn, array) != 2 || array[0] != 5) {
            this.bindV4(cmdIn, cmdOut, obj.getAddress(), obj.getPort());
            return;
        }
        if (array[1] == -1) {
            throw new SocketException("SOCKS : No acceptable methods");
        }
        if (!this.authenticate(array[1], cmdIn, cmdOut)) {
            throw new SocketException("SOCKS : authentication failed");
        }
        cmdOut.write(5);
        cmdOut.write(2);
        cmdOut.write(0);
        int port = obj.getPort();
        if (obj.isUnresolved()) {
            cmdOut.write(3);
            cmdOut.write(obj.getHostName().length());
            try {
                cmdOut.write(obj.getHostName().getBytes("ISO-8859-1"));
            }
            catch (UnsupportedEncodingException ex4) {
                assert false;
            }
            cmdOut.write(port >> 8 & 0xFF);
            cmdOut.write(port >> 0 & 0xFF);
        }
        else if ((obj.getAddress() instanceof Inet4Address)) {
            byte[] address2 = obj.getAddress().getAddress();
            cmdOut.write(1);
            cmdOut.write(address2);
            cmdOut.write(port >> 8 & 0xFF);
            cmdOut.write(port >> 0 & 0xFF);
            cmdOut.flush();
        }
        else if ((obj.getAddress() instanceof Inet6Address)) {
            byte[] address = obj.getAddress().getAddress();
            cmdOut.write(4);
            cmdOut.write(address);
            cmdOut.write(port >> 8 & 0xFF);
            cmdOut.write(port >> 0 & 0xFF);
            cmdOut.flush();
        }
        else {
            this.cmdsock.close();
            throw new SocketException("unsupported address type : " + obj);
        }
        byte[] array2 = new byte[4];
        this.readSocksReply(cmdIn, array2);
        Object o2 = null;
        switch (array2[1]) {
            case 0: {
                switch (array2[3]) {
                    case 1: {
                        byte[] array3 = new byte[4];
                        if (this.readSocksReply(cmdIn, array3) != 4) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        byte[] array4 = new byte[2];
                        if (this.readSocksReply(cmdIn, array4) != 2) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        this.external_address = new InetSocketAddress(new Inet4Address("", array3), ((array4[0] & 0xFF) << 8) + (array4[1] & 0xFF));
                        break;
                    }
                    case 3: {
                        int n = array2[1];
                        byte[] bytes = new byte[n];
                        if (this.readSocksReply(cmdIn, bytes) != n) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        byte[] array5 = new byte[2];
                        if (this.readSocksReply(cmdIn, array5) != 2) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        this.external_address = new InetSocketAddress(new String(bytes), ((array5[0] & 0xFF) << 8) + (array5[1] & 0xFF));
                        break;
                    }
                    case 4: {
                        int n2 = array2[1];
                        byte[] array6 = new byte[n2];
                        if (this.readSocksReply(cmdIn, array6) != n2) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        byte[] array7 = new byte[2];
                        if (this.readSocksReply(cmdIn, array7) != 2) {
                            throw new SocketException("Reply from SOCKS server badly formatted");
                        }
                        this.external_address = new InetSocketAddress(new Inet6Address("", array6), ((array7[0] & 0xFF) << 8) + (array7[1] & 0xFF));
                        break;
                    }
                }
                break;
            }
            case 1: {
                o2 = new SocketException("SOCKS server general failure");
                break;
            }
            case 2: {
                o2 = new SocketException("SOCKS: Bind not allowed by ruleset");
                break;
            }
            case 3: {
                o2 = new SocketException("SOCKS: Network unreachable");
                break;
            }
            case 4: {
                o2 = new SocketException("SOCKS: Host unreachable");
                break;
            }
            case 5: {
                o2 = new SocketException("SOCKS: Connection refused");
                break;
            }
            case 6: {
                o2 = new SocketException("SOCKS: TTL expired");
                break;
            }
            case 7: {
                o2 = new SocketException("SOCKS: Command not supported");
                break;
            }
            case 8: {
                o2 = new SocketException("SOCKS: address type not supported");
                break;
            }
        }
        if (o2 != null) {
            cmdIn.close();
            cmdOut.close();
            this.cmdsock.close();
            this.cmdsock = null;
            throw o2;
        }
        this.cmdIn = cmdIn;
        this.cmdOut = cmdOut;
        return;
    }
    
    protected void acceptFrom(SocketImpl socketImpl, InetSocketAddress inetSocketAddress) throws IOException {
        if (this.cmdsock == null) {
            return;
        }
        InputStream cmdIn = this.cmdIn;
        this.socksBind(inetSocketAddress);
        cmdIn.read();
        int read = cmdIn.read();
        cmdIn.read();
        SocketException ex = null;
        InetSocketAddress external_address = null;
        switch (read) {
            case 0: {
                switch (cmdIn.read()) {
                    case 1: {
                        byte[] array = new byte[4];
                        this.readSocksReply(cmdIn, array);
                        external_address = new InetSocketAddress(new Inet4Address("", array), (cmdIn.read() << 8) + cmdIn.read());
                        break;
                    }
                    case 3: {
                        byte[] bytes = new byte[cmdIn.read()];
                        this.readSocksReply(cmdIn, bytes);
                        external_address = new InetSocketAddress(new String(bytes), (cmdIn.read() << 8) + cmdIn.read());
                        break;
                    }
                    case 4: {
                        byte[] array2 = new byte[16];
                        this.readSocksReply(cmdIn, array2);
                        external_address = new InetSocketAddress(new Inet6Address("", array2), (cmdIn.read() << 8) + cmdIn.read());
                        break;
                    }
                }
                break;
            }
            case 1: {
                ex = new SocketException("SOCKS server general failure");
                break;
            }
            case 2: {
                ex = new SocketException("SOCKS: Accept not allowed by ruleset");
                break;
            }
            case 3: {
                ex = new SocketException("SOCKS: Network unreachable");
                break;
            }
            case 4: {
                ex = new SocketException("SOCKS: Host unreachable");
                break;
            }
            case 5: {
                ex = new SocketException("SOCKS: Connection refused");
                break;
            }
            case 6: {
                ex = new SocketException("SOCKS: TTL expired");
                break;
            }
            case 7: {
                ex = new SocketException("SOCKS: Command not supported");
                break;
            }
            case 8: {
                ex = new SocketException("SOCKS: address type not supported");
                break;
            }
        }
        if (ex != null) {
            this.cmdIn.close();
            this.cmdOut.close();
            this.cmdsock.close();
            this.cmdsock = null;
            throw ex;
        }
        if ((socketImpl instanceof SocksSocketImpl)) {
            ((SocksSocketImpl)socketImpl).external_address = external_address;
        }
        if ((socketImpl instanceof PlainSocketImpl)) {
            PlainSocketImpl plainSocketImpl = (PlainSocketImpl)socketImpl;
            plainSocketImpl.setInputStream((SocketInputStream)cmdIn);
            plainSocketImpl.setFileDescriptor(this.cmdsock.getImpl().getFileDescriptor());
            plainSocketImpl.setAddress(this.cmdsock.getImpl().getInetAddress());
            plainSocketImpl.setPort(this.cmdsock.getImpl().getPort());
            plainSocketImpl.setLocalPort(this.cmdsock.getImpl().getLocalPort());
        }
        else {
            socketImpl.fd = this.cmdsock.getImpl().fd;
            socketImpl.address = this.cmdsock.getImpl().address;
            socketImpl.port = this.cmdsock.getImpl().port;
            socketImpl.localport = this.cmdsock.getImpl().localport;
        }
        this.cmdsock = null;
        return;
    }
    
    @Override
    protected InetAddress getInetAddress() {
        if (this.external_address != null) {
            return this.external_address.getAddress();
        }
        return super.getInetAddress();
    }
    
    @Override
    protected int getPort() {
        if (this.external_address != null) {
            return this.external_address.getPort();
        }
        return super.getPort();
    }
    
    @Override
    protected int getLocalPort() {
        if (this.socket != null) {
            return super.getLocalPort();
        }
        if (this.external_address != null) {
            return this.external_address.getPort();
        }
        return super.getLocalPort();
    }
    
    @Override
    protected void close() throws IOException {
        if (this.cmdsock != null) {
            this.cmdsock.close();
        }
        this.cmdsock = null;
        super.close();
    }
    
    private String getUserName() {
        String property = "";
        if (this.applicationSetProxy) {
            try {
                property = System.getProperty("user.name");
            }
            catch (SecurityException ex) {}
        }
        else {
            property = (String)AccessController.doPrivileged(new GetPropertyAction("user.name"));
        }
        return property;
    }
    
    static void access$000(SocksSocketImpl socksSocketImpl, String s, int n, int n2) throws IOException {
        socksSocketImpl.superConnectServer(s, n, n2);
    }
}
